/*
 * Copyright (c) 2018 Meltytech, LLC
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import QtQuick.Controls 1.1
import QtQuick.Layouts 1.1
import Shotcut.Controls 1.0

Item {
    
    property string noise: '0'
    property double noiseDefault: 0.200
    property bool blockUpdate: true
	property real maxFilterPercent: 100.00
    property real maxUserPercent: 100.00
    property var defaultParameters: [noise]
    property double startValue: 0.0
    property double middleValue: 0.5
    property double endValue: 0.0
   
    width: 250
    height: 100
    
    Component.onCompleted: {
        if (filter.isNew) {
            filter.set(noise, noiseDefault)   
            filter.savePreset(preset.defaultParameters)
        } else {
            middleValue = filter.getDouble(noise, filter.animateIn)
            if (filter.animateIn > 0)
                startValue = filter.getDouble(noise, 0)
            if (filter.animateOut > 0)
                endValue = filter.getDouble(noise, filter.duration - 1)
        }
        setControls()
    }

    function getPosition() {
        return Math.max(producer.position - (filter.in - producer.in), 0)
    }

    function setControls() {
        var position = getPosition()
        blockUpdate = true
        noiseSlider.value = filter.getDouble(noise, position) * maxFilterPercent
        blockUpdate = false
        noiseSlider.enabled = position <= 0 || (position >= (filter.animateIn - 1) && position <= (filter.duration - filter.animateOut)) || position >= (filter.duration - 1)
    }

    function updateFilter(position) {
        if (blockUpdate) return
        var value = noiseSlider.value / maxFilterPercent

        if (position !== null) {
            if (position <= 0 && filter.animateIn > 0)
                startValue = value
            else if (position >= filter.duration - 1 && filter.animateOut > 0)
                endValue = value
            else
                middleValue = value
        }

        if (filter.animateIn > 0 || filter.animateOut > 0) {
            filter.resetProperty(noise)
            noiseKeyframesButton.checked = false
            if (filter.animateIn > 0) {
                filter.set(noise, startValue, 0)
                filter.set(noise, middleValue, filter.animateIn - 1)
            }
            if (filter.animateOut > 0) {
                filter.set(noise, middleValue, filter.duration - filter.animateOut)
                filter.set(noise, endValue, filter.duration - 1)
            }
        } else if (!noiseKeyframesButton.checked) {
            filter.resetProperty(noise)
            filter.set(noise, middleValue)
        } else if (position !== null) {
            filter.set(noise, value, position)
        }
    }

    GridLayout {
        anchors.fill: parent
        anchors.margins: 8
        columns: 4

        Label {
            text: qsTr('Preset')
            Layout.alignment: Qt.AlignRight
        }
        Preset {
            id: preset
			Layout.columnSpan: 3
            parameters: defaultParameters
            onBeforePresetLoaded: filter.resetProperty(noise)
            onPresetSelected: {
                setControls()
                noiseKeyframesButton.checked = filter.keyframeCount(noise) > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
                middleValue = filter.getDouble(noise, filter.animateIn)
                if (filter.animateIn > 0)
                    startValue = filter.getDouble(noise, 0)
                if (filter.animateOut > 0)
                    endValue = filter.getDouble(noise, filter.duration - 1)
            }
        }

        Label {
            text: qsTr('Noise')
            Layout.alignment: Qt.AlignRight
        }
        SliderSpinner {
            id: noiseSlider
            minimumValue: 0.00
            maximumValue: maxUserPercent
			stepSize: 0.01
            decimals: 2
			suffix: ' %'
            onValueChanged: updateFilter(getPosition())
        }
        UndoButton {
            onClicked: noiseSlider.value = noiseDefault * maxFilterPercent
        }
        KeyframesButton {
            id: noiseKeyframesButton
            checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount(noise) > 0
            onToggled: {
                var value = noiseSlider.value / maxFilterPercent
                if (checked) {
                    blockUpdate = true
                    filter.clearSimpleAnimation(noise)
                    blockUpdate = false
                    filter.set(noise, value, getPosition())
                } else {
                    filter.resetProperty(noise)
                    filter.set(noise, value)
                }
            }
        }
  
        Item {
            Layout.fillHeight: true;
        }
    }
    // Connections {
       // target: producer
      //  onPositionChanged: setControls()
    // }
  }